function [n, fn, alpha1, alpha2] = dfa_edit(t, sig)


load DFA_defaults

% Define input
p = inputParser;
p.KeepUnmatched = true;
p.addRequired('t', @(x) isnumeric(x) && ~isscalar(x));
p.addRequired('sig', @(x) isnumeric(x) && ~isscalar(x));
p.addParameter('n_min',  DEFAULT_NMIN, @(x) isnumeric(x) && isscalar(x));
p.addParameter('n_max',  DEFAULT_NMAX, @(x) isnumeric(x) && isscalar(x));
p.addParameter('n_incr',  DEFAULT_N_INCR, @(x) isnumeric(x) && isscalar(x));
p.addParameter('alpha1_range',  DEFAULT_ALPHA1_RANGE, @(x) isnumeric(x) && numel(x) == 2);
p.addParameter('alpha2_range',  DEFAULT_ALPHA2_RANGE, @(x) isnumeric(x) && numel(x) == 2);

% Get input
p.parse(t, sig);
n_min = p.Results.n_min;
n_max = p.Results.n_max;
n_incr = p.Results.n_incr;
alpha1_range = p.Results.alpha1_range;
alpha2_range = p.Results.alpha2_range;

%% Initializations

% Integrate the signal without mean
nni_int = cumsum(sig - mean(sig));

N = length(nni_int);

% Create n-axis (box-sizes)
% If n_incr is less than 1 we interpret it as the ratio of a geometric
% series on box sizes. This should produce box sizes identical to the
% PhysioNet DFA implmentation.
if n_incr < 1
    M = log2(n_max/n_min) * (1/n_incr);
    n = unique(floor(n_min.*(2^n_incr).^(0:M)+0.5));
else
    n = n_min:n_incr:n_max;
end

fn = ones(n_max, 1) * NaN;

%% DFA
for nn = n
    % Calculate the number of windows we need for the current n
    num_win = floor(N/nn);

    % Break the signal into num_win windows of n samples each
    sig_windows = reshape(nni_int(1:nn*num_win), nn, num_win);
    t_windows  = reshape(t(1:nn*num_win), nn, num_win);
    sig_regressed = zeros(size(sig_windows));

    % Perform linear regression in each window
    for ii = 1:num_win
        y = sig_windows(:, ii);
        x = [ones(nn, 1), t_windows(:, ii)];
        b = x\y;
        yn = x * b;
        sig_regressed(:, ii) = yn;
    end

    % Calculate F(n), the value of the DFA for the current n
    fn(nn) = sqrt ( 1/N * sum((sig_windows(:) - sig_regressed(:)).^2) );
end

% Find the indices of all the DFA values we calculated
fn = fn(n);
n  = n';

% If fn is zero somewhere (might happen in the small scales if there's not
% enough data points there) set it to some small constant to prevent
% log(0)=-Inf.
fn(fn<1e-9) = 1e-9;

%% Scaling exponent, alpha

% Find DFA values in each of the alpha ranges
alpha1_idx = find(n >= alpha1_range(1) & n <= alpha1_range(2));
alpha2_idx = find(n >= alpha2_range(1) & n <= alpha2_range(2));

% Fit a line to the log-log DFA in each alpha range
fn_log = log10(fn);
n_log = log10(n);
fit_alpha1 = polyfit(n_log(alpha1_idx), fn_log(alpha1_idx), 1);
fit_alpha2 = polyfit(n_log(alpha2_idx), fn_log(alpha2_idx), 1);

% Save the slopes of the lines
alpha1 = fit_alpha1(1);
alpha2 = fit_alpha2(1);

end

